#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  config.pm - A simple `name=value` configuration file reader.
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/


package gnump3d::config;  # must live in config.pm
require Exporter;
use strict;
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

# set the version for version checking
$VERSION     = "0.01";

@ISA         = qw(Exporter);
@EXPORT      = qw( &readConfig &getConfig &configFile &getHash &configUsesEnvironment);
%EXPORT_TAGS = ( );

# your exported package globals go here,
# as well as any optionally exported functions
@EXPORT_OK   = ( ) ;


#
#  The hash storing the name and values we read from the
# configuration file.
#
my %HASH     = ();


#
#  The file that we, last, read our configuration values from.
#
my $CONFIG_FILE = "";

#
#  If this is set then environmental variables may override the
# configuration file.
#
my $USE_ENVIRONMENT = 0;



#
#
#  Read the configuration file.
#
#
sub readConfig($)
{
    my ($file) = (@_);

    #
    # Save the name of the configuratione file.
    #
    $CONFIG_FILE = $file;


    open( FILY, "<$file" ) or die "Cannot open file: $file - $!";
    my @lines = <FILY>;
    close( FILY );


    my $fieldCount = -1;
    my $lineCount  = 0;

    foreach my $line ( @lines )
    {
	# Skip lines beginning with comments
        next if ( $line =~ /^([ \t]*)\#/ );

        # Skip blank lines
        chomp( $line );
        next if ( length( $line ) < 1 );

        # Strip trailing comments.
        if ( $line =~ /(.*)\#(.*)/ )
        {
            $line = $1;
        }

        # Find variable settings
        if ( $line =~ /([^=]+)=([^\n]+)/ )
	{
	    my $key = $1;
	    my $val = $2;

	    # Strip leading and trailing whitespace.
	    $key =~ s/^\s+//;
	    $key =~ s/\s+$//;
	    $val =~ s/^\s+//;
	    $val =~ s/\s+$//;

	    # Store value.
	    $HASH{ $key } = $val;
	}
    }
}


#
#  Allow the caller to specify if environmental variables may be used
# to override the configuration file settings.
#
#  Default is not to allow this.
#
sub configUsesEnvironment( $ )
{
    my ( $enabled ) = ( @_ );
    $USE_ENVIRONMENT = $enabled;
}


#
#
#  Return a configuration setting from the config file.
#
#
sub getConfig( $ )
{
    my ( $key ) = ( @_ );
    my $value   = undef;

    if ( $USE_ENVIRONMENT )
    {
	$value = $ENV{$key};
    }

    if ( !defined( $value ) )
    {
	$value = $HASH{ $key };
    }
    return( $value );
}



#
# Return the name of the configuration file we read.
#
sub configFile( )
{
    return( $CONFIG_FILE );
}


#
#  Return the hash of all our configuration options to
# the caller.
#
sub getHash()
{
    return( %HASH );
}


#
# End of module
#
1;
