#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  bugs.pm  - Allow users to make bug reports.
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# Version:
#  $Id: bug.pm,v 1.7 2003/01/26 23:44:59 stevekemp Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/
#
#



#
#  Return the author of this plugin.
#
sub getAuthor()
{
    return( 'Steve Kemp <steve@steve.org.uk>' );
}


#
#  Return the version of this plugin.
#
sub getVersion()
{
    my $REVISION      = '$Id: bug.pm,v 1.7 2003/01/26 23:44:59 stevekemp Exp $';
    my $VERSION       = "";
    $VERSION = join (' ', (split (' ', $REVISION))[1..3]);
    $VERSION =~ s/,v\b//;
    $VERSION =~ s/(\S+)$/($1)/;

    return( $VERSION );
}



#
#  Handle requests to this plugin.
#
sub handlePath( )
{
    my ( $uri ) = (@_);

    my $header   = &getHTTPHeader( 200, "text/html" );
    &sendData( $data, $header );

    my $text = &getBugsForm( $ARGUMENTS{"theme"});
    &sendData( $data, $text );

    close( $data );
    exit;
}



#
#  Read and return the bug reporting form to the caller.
sub getBugsForm( $ )
{
    my ($theme) = (@_);
    my $text    = "";

    my @template = &getThemeFile( $theme, "plugin.html" );
    foreach my $line (@template )
    {
	#
	# Make global substitutions.
	#
	$line =~ s/\$HOSTNAME/$host/g;
	$line =~ s/\$VERSION/$VERSION/g;
	$line =~ s/\$DIRECTORY/\/bug\//g;
	$line =~ s/\$HEADING/GNUMP3d Bug Reporting Page/g;
	$line =~ s/\$TITLE/GNUMP3d Bug Reporting Page/g;

	#
	# Now handle the special sections.
	#
	if ( $line =~ /(.*)\$BANNER(.*)/ )
	{
	    # Insert banner;
	    my $pre  = $1;
	    my $post = $2;

	    $text .= $pre;
	    $text .= &getBanner( "/bug/" );
	    $text .= $post;
	}
	elsif ( $line =~ /(.*)\$TEXT(.*)/ )
	{
	    my $pre  = $1;
	    my $post = $2;
	    $SIG{CHLD} = "DEFAULT";
	    $text .= $pre;

	    #
	    # My bug reporting CGI script.
	    #
	    my $BUG_REPORT_URL =  "http://www.steve.org.uk/cgi-bin/gnump3dbug/report.cgi";
	    my %CONFIG_HASH = &getHash();
	    my $BUG_CONFIG  = "";

	    # Build up a string containing the server's configuration options.
	    foreach my $key ( sort ( keys( %CONFIG_HASH ) ) )
	    {
		$BUG_CONFIG .= $key . " = '" . $CONFIG_HASH{$key} . "'\n";
	    }


	    $text .=<<E_O_BUG;
<p>&nbsp;This form allows you to easily report a bug with the <a href="http://www.gnump3d.org/">GNUMP3d</a> server.</P>
<p>&nbsp;To try to make this a simple a process as possible some information has already been gleaned from your system, and entered here.</p>
<P>&nbsp;Please try to describe your problem as accurately as you can, this should make solving your problem easier for me.</p>
<table width='100%'>
<form action='" . $BUG_REPORT_URL . "' method='post'>
<table>
<tr><td><b>Your Name</b></td><td><input type='text' size='50' name='name' /></td></tr>
<tr><td><b>Email Address</b></td><td><input type='text' size='50' name='email' /></td></tr>
<tr><td colspan='2'>(You don't need to fill in your address, but if you do I'll be able to mail you for further information, or to let you know when the problem has been resolved).</td></tr>
<!--  Bug entry space -->
<tr><td valign='top'><b>Please Describe Your Bug Here</b></td><td><textarea name='problem' cols='50' rows='25'>
</textarea></td></tr>
<tr><td colspan='2'><p>&nbsp;The following values have been automatically determined and entered for your convienence.</P>
<p>&nbsp;If you believe any of the values conpromise your privacy, or reveal any sensitive information, feel free to remove them.  (There are no other hidden variables being used, only the information contained in these public fields will be sent).</p>
<p>&nbsp;Please note that leaving them in could be genuinely useful to solving your problem though.</p>
</td></tr>
<tr><td><b>GNUMP3d release</b></td><td><input type='text' size='50' name='release' value='$VERSION'/></td></tr>
<tr><td><b>Current User-Agent</b></td><td><input type='text' size='50' name='user-agent' value='$USER_AGENT'/></td></tr>
<tr><td><b>Current IP Address</b></td><td><input type='text' size='50' name='ip-address' value='$connected_address'/></td></tr>
<tr><td><b>Requested URI</b></td><td><input type='text' size='50' name='uri' value='$REQUEST'/></td></tr>
<tr><td><b>Server-Name</b></td><td><input type='text' size='50' name='server-name' value='http://$host/'/></td></tr>
<tr><td><b>Perl Version</b></td><td><input type='text' size='50' name='perl-version' value='$]'/></td></tr>
<tr><td><b>OS Name</b></td><td><input type='text' size='50' name='os-name' value='$OSNAME'/></td></tr>
<!--  Configuration Options -->
<tr><td valign='top'><b>Your Server Configuration</b></td><td><textarea name='gnump3d.conf' cols='50' rows='25'>
$BUG_CONFIG
</textarea></td></tr>

<tr><td>&nbsp;</td><td><input type='submit' value='Report Bug' /></td></tr>
</table>\n
E_O_BUG
	    $text .= $post;
	    $SIG{CHLD} = "IGNORE";
	}
	else
	{
	    $text .= $line;
	}
    }

    return( $text );
}
