#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  last.pm  - Display the most recent tracks which have been served.
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# Version:
#  $Id: last.pm,v 1.6 2003/01/26 23:44:59 stevekemp Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/
#
#



#
#  Return the author of this plugin.
#
sub getAuthor()
{
    return( 'Steve Kemp <steve@steve.org.uk>' );
}


#
#  Return the version of this plugin.
#
sub getVersion()
{
    my $REVISION      = '$Id: last.pm,v 1.6 2003/01/26 23:44:59 stevekemp Exp $';
    my $VERSION       = "";
    $VERSION = join (' ', (split (' ', $REVISION))[1..3]);
    $VERSION =~ s/,v\b//;
    $VERSION =~ s/(\S+)$/($1)/;

    return( $VERSION );
}



#
#  Handle requests to this plugin.
#
sub handlePath( )
{
    my ( $uri ) = (@_);

    my $header   = &getHTTPHeader( 200, "text/html" );
    &sendData( $data, $header );

    my $text = &handleLast( $ARGUMENTS{"theme"});
    &sendData( $data, $text );

    close( $data );
    exit;
}



#
#  Display a list of the most recent songs played.
#
sub handleLast( $ )
{
    my ($theme) = (@_);
    my $text  ="";

    my $config = &configFile();

    my @template = &getThemeFile( $theme, "plugin.html" );
    foreach my $line (@template )
    {
	#
	# Make global substitutions.
	#
	$line =~ s/\$HOSTNAME/$host/g;
	$line =~ s/\$VERSION/$VERSION/g;
	$line =~ s/\$DIRECTORY/\/last\//g;
	$line =~ s/\$HEADING/Most Recent Tracks Played/g;
	$line =~ s/\$TITLE/Most Recent Tracks Played/g;

	#
	# Now handle the special sections.
	#
	if ( $line =~ /(.*)\$BANNER(.*)/ )
	{
	    # Insert banner;
	    my $pre  = $1;
	    my $post = $2;

	    $text .= $pre;
	    $text .= &getBanner( "/last/" );
	    $text .= $post;
	}
	elsif ( $line =~ /(.*)\$TEXT(.*)/ )
	{
	    my $pre  = $1;
	    my $post = $2;
	    $SIG{CHLD} = "DEFAULT";
	    $text .= $pre;

	    my $popularsongs = `$STATSPROG --config=$config $STATSARGS --plain --last`;

	    my @songs = split( /\n/, $popularsongs );
	    foreach my $track ( @songs )
	    {
		chomp( $track );
		next if ( $track eq "Song" );
		$text .= "<tr><td width=\"10%\">&nbsp;</td><td><a href=\"$track\">";
		$text .= &getSongDisplay( $ROOT . "/" . $track, $song_format );
		$text .= "</a></td></tr>\n";
	    }


	    $text .= $post;
	    $SIG{CHLD} = "IGNORE";
	}
	else
	{
	    $text .= $line;
	}
    }

    return( $text );
}

