#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  playlist.pm - Allow users to create their own custom playlists.
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# Version:
#  $Id: playlist.pm,v 1.7 2003/01/30 11:22:36 stevekemp Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/
#
#



#
#  Return the author of this plugin.
#
sub getAuthor()
{
    return( 'Steve Kemp <steve@steve.org.uk>' );
}


#
#  Return the version of this plugin.
#
sub getVersion()
{
    my $REVISION      = '$Id: playlist.pm,v 1.7 2003/01/30 11:22:36 stevekemp Exp $';
    my $VERSION       = "";
    $VERSION = join (' ', (split (' ', $REVISION))[1..3]);
    $VERSION =~ s/,v\b//;
    $VERSION =~ s/(\S+)$/($1)/;

    return( $VERSION );
}



#
#  Handle requests to this plugin.
#
sub handlePath( )
{
    #
    # Should we play the results of the form?
    #
    if ( defined $ARGUMENTS{ "play" }  )
    {
	my $playlist = "";
	foreach my $option ( keys %ARGUMENTS )
	{
	    if ( $option =~ /^dir/ )
	    {
		my $dir = $ARGUMENTS{ $option };
		my $dirPlaylist = &getPlaylist( $dir . "/all.m3u" );
		$playlist .= $dirPlaylist;
	    }
	}

	#
	#   Remove the play argument from the cookies we send back
	# in our header.
	#
	#  If we don't do this subsequent requests to the /play/
	# URL will not view the form.
	#
	delete( $ARGUMENTS{ "play" } );

	my $header   = &getHTTPHeader( 200, "audio/x-mpegurl" );
	&sendData($data, $header );
	&sendData($data, $playlist );
	close($data);
	exit;
    }


    my $header   = &getHTTPHeader( 200, "text/html" );
    &sendData( $data, $header );
    my $text = &showCustomPlaylistForm( $ARGUMENTS{"theme"});
    &sendData( $data, $text );
    close( $data );
    exit;
}


#
#  Show the list of all the top level directories within the music
# archive and allow people to choose between them.
#
sub showCustomPlaylistForm( $ )
{
    my ($theme) = (@_);
    my $text  ="";

    my @template = &getThemeFile( $theme, "plugin.html" );
    foreach my $line (@template )
    {
	#
	# Make global substitutions.
	#
	$line =~ s/\$HOSTNAME/$host/g;
	$line =~ s/\$VERSION/$VERSION/g;
	$line =~ s/\$DIRECTORY/\/themes\//g;
	$line =~ s/\$HEADING/Custom Playlist Generation/g;
	$line =~ s/\$TITLE/Custom Playlist Generation/g;

	#
	# Now handle the special sections.
	#
	if ( $line =~ /(.*)\$BANNER(.*)/ )
	{
	    # Insert banner;
	    my $pre  = $1;
	    my $post = $2;

	    $text .= $pre;
	    $text .= &getBanner( "/playlist/" );
	    $text .= $post;
	}
	elsif ( $line =~ /(.*)\$TEXT(.*)/ )
	{
	    my $pre  = $1;
	    my $post = $2;
	    my @dirs = &dirsInDir( $ROOT );

	    $text .= "<form action=\"/playlist/\">\n";
	    $text .= "<input type=\"hidden\" name=\"play\" value=\"1\">\n";

	    $text .= $pre;
	    my $count = 0;
	    foreach my $name ( @dirs )
	    {
		my $checked = "";
		if ( defined( $ARGUMENTS{"dir$count" } ) )
		  {
		      $checked = "checked";
		  }
		$text .= "<input type='checkbox' name='dir$count' value='/$name' $checked> <a href=\"/$name/\">$name</a><br />";
		$count += 1;

	    }
	    $text .= "<input type=\"reset\" value=\"Clear All\"> <input type=\"submit\" value=\"Play\">\n";
	    $text .= "</form>\n";
	    $text .= $post;
	}
	else
	{
	    $text .= $line;
	}
    }

    return( $text );
}
